import json
import random
import subprocess
import sys
from pathlib import Path
from typing import Any

import tqdm
from git import Repo

from mentat import Mentat
from mentat.config import Config
from mentat.errors import SampleError
from mentat.git_handler import get_git_diff
from mentat.parsers.git_parser import GitParser
from mentat.sampler.sample import Sample
from mentat.sampler.utils import apply_diff_to_repo, get_active_snapshot_commit, setup_repo
from mentat.session_context import SESSION_CONTEXT


def setup_sample(
    sample: Sample, cwd: Path | str | None, skip_test_exec: bool = False
) -> tuple[Repo, Path, str, str | None]:
    setup_commit = sample.environment_setup_commit or sample.merge_base
    repo = setup_repo(
        url=sample.repo,
        cwd=cwd,
        commit=setup_commit,
        diff_merge_base=sample.diff_merge_base,
        diff_active=sample.diff_active,
    )
    cwd = Path(repo.working_dir)

    test_executable = None
    if not skip_test_exec and (sample.FAIL_TO_PASS or sample.PASS_TO_PASS):
        # If there's an environment_setup_commit, this is what it's needed for.
        try:
            test_executable = get_test_executable(Path(repo.working_dir))
        except SampleError as e:
            print(f"Error setting up virtual environment: {e}")
            print("Using default python executable instead.")
    if not test_executable:
        test_executable = sys.executable

    if sample.environment_setup_commit and sample.merge_base:
        # SWE-Bench samples have an environmental_setup_commit (above),
        # then a merge_base to checkout.
        repo.git.reset("--hard")
        repo.git.checkout(sample.merge_base)
        commit_active = sample.merge_base
    else:
        # Mentat Samples have an active diff which was set in setup_repo,
        # so here create a snapshot commit (to generate diff_edit against later)
        commit_active = get_active_snapshot_commit(repo)

    return repo, cwd, test_executable, commit_active


async def run_sample(sample: Sample, cwd: Path | str | None = None, config: Config | None = None) -> dict[str, Any]:
    """Run a sample using Mentat and return the resulting diff"""

    repo, cwd, test_executable, commit_active = setup_sample(sample, cwd)

    # Run sample in PythonClient
    paths = list[Path]()
    for a in sample.context:
        paths.append(Path(a))
    mentat = Mentat(cwd=cwd, paths=paths, config=config or Config())
    await mentat.startup()
    session_context = SESSION_CONTEXT.get()
    conversation = session_context.conversation
    cost_tracker = session_context.cost_tracker
    for msg in sample.message_history:
        if msg["role"] == "user":
            conversation.add_user_message(msg["content"])
        elif msg["role"] == "assistant":
            parsed_llm_response = GitParser().parse_llm_response(msg["content"])
            content = session_context.config.parser.file_edits_to_llm_message(parsed_llm_response)
            conversation.add_model_message(content, [], parsed_llm_response)
        else:
            raise SampleError(f"Invalid role found in message_history: {msg['role']}")
    prompt = sample.message_prompt
    if sample.hint_text:
        prompt += f"\n{80 * '-'}\nHint Text:\n{sample.hint_text}"
    await mentat.call_mentat_auto_accept(prompt)
    await mentat.shutdown()

    # Get the diff between pre- and post-edit
    transcript_messages = conversation.literal_messages.copy()

    message_eval = str(transcript_messages[-1].get("message", ""))
    diff_eval = get_git_diff(commit_active or "HEAD", cwd=cwd)

    test_results = None
    test_passed = None
    if sample.test_patch:
        apply_diff_to_repo(sample.test_patch, repo)
    if sample.FAIL_TO_PASS:
        tests = json.loads(sample.FAIL_TO_PASS)
        total = len(tests)
        passed = 0
        errors = list[dict[str, str]]()
        for test in tests:
            _passed, _error = get_test_result(test, cwd, test_executable)
            if _passed:
                passed += 1
            if _error:
                errors.append({"test": test, "error": _error})
        test_results = {
            "passed": passed,
            "total": total,
            "passed_percent": passed / total * 100,
            # "errors": errors,  # Too big, but useful for debugging
        }
        test_passed = passed == total

    return {
        "id": sample.id,
        "message_eval": message_eval,
        "diff_eval": diff_eval,
        "cost": cost_tracker.total_cost,
        "tokens": cost_tracker.total_tokens,
        "transcript": {
            "id": sample.id,
            "messages": transcript_messages,
        },
        "test_eval_results": test_results,
        "test_eval_passed": test_passed,
    }


test_requirements_for_repo = {
    "pvlib-python": [
        "setuptools",
        "pytest",
        "pytest-cov",
        "pytest-mock",
        "requests-mock",
        "pytest-timeout",
        "pytest-rerunfailures",
        "pytest-remotedata",
    ],
    "pydicom": ["setuptools", "pytest"],
    "sqlfluff": [
        "setuptools",
        "pytest",
        "pytest-cov",
        "pytest-mock",
        "Jinja2",
        "oyaml",
    ],
    "pyvista": [
        "setuptools",
        "pytest",
        "ipython",
        "ipywidgets",
        "ipykernel",
        "tqdm",
    ],
    "astroid": [
        "setuptools",
        "pytest",
        "attrs",
        "types-attrs",
        "nose",
        "numpy",
        "python-dateutil",
        "types-python-dateutil",
        "six",
        "types-six",
    ],
    "marshmallow": [
        "setuptools",
        "pytest",
        "pytz",
        "simplejson",
    ],
}


def get_test_executable(cwd: Path) -> str:
    """Rebuild every time with the latest setup."""

    venv_dir = cwd / ".venv"
    repo_name = cwd.name

    try:
        python_executable = "python3" if sys.platform != "win32" else "python"
        subprocess.run([python_executable, "-m", "venv", str(venv_dir)], check=True, cwd=cwd, capture_output=True)
    except Exception as e:
        raise SampleError(f"Error creating virtual environment: {e}")

    # Install as a pip module
    try:
        output = subprocess.run(
            [venv_dir / "bin" / "pip", "install", "-e", "."], check=True, cwd=cwd, capture_output=True
        )
        if output.returncode != 0:
            raise SampleError(f"Error installing sample as a pip module: {output.stderr}")
    except Exception as e:
        raise SampleError(f"Error installing sample as a pip module: {e}")

    # Requirements are hard-coded by repo
    if repo_name not in test_requirements_for_repo:
        raise SampleError(f"No requirements found for repo '{repo_name}'")
    requirements = test_requirements_for_repo[repo_name]

    # Install them all with pip
    try:
        output = subprocess.run(
            [venv_dir / "bin" / "pip", "install", *list(requirements)], check=True, cwd=cwd, capture_output=True
        )
        if output.returncode != 0:
            raise SampleError(f"Error installing requirements: {output.stderr}")
    except Exception as e:
        raise SampleError(f"Error installing requirements: {e}")

    return str(venv_dir / "bin" / "python")


def get_test_result(test: str, cwd: Path, test_executable: str) -> tuple[bool, str]:
    passed, error = False, ""
    command = [test_executable, "-m", "pytest"]
    if "[" in test:
        # Some tests include parameters, like "..is_valid[3.1415]".
        # Running '-k' over the whole suite is very slow.
        path, params = test.split("[", 1)
        params = params[:-1]  # Remove trailing ']'
        command += [path, "-k", params]
    else:
        command += [test]
    try:
        output = subprocess.run(
            command,
            capture_output=True,
            text=True,
            cwd=cwd,
        )
        if (output.returncode != 0 and output.stderr) or not output.stdout:
            raise SampleError(f"Test command failed: {output.stderr}")

        # Starting from the end, find the first line that contains "passed" or "failed"
        lines = output.stdout.splitlines()
        result_line = next(
            line for line in reversed(lines) if any(key in line for key in {"passed", "failed", "skipped"})
        )
        _passed = "passed" in result_line or "skipped" in result_line
        _failed = "failed" in result_line
        if _passed == _failed:
            raise SampleError(f"Could not determine test result from line: {result_line}")
        passed = _passed
        if _failed:
            raise SampleError("Test failed:\n" + "\n".join(lines))
    except (SampleError, StopIteration, Exception) as e:
        error = str(e)
    return passed, error


def validate_test_fields(sample: Sample) -> dict[str, Any]:
    test_results: dict[str, Any] = {
        "PASS_TO_PASS": {"passed": 0, "total": 0, "errors": []},
        "FAIL_TO_PASS_PRE": {"passed": 0, "total": 0, "errors": []},
        "FAIL_TO_PASS_POST": {"passed": 0, "total": 0, "errors": []},
    }

    if not sample.FAIL_TO_PASS and not sample.PASS_TO_PASS:
        return test_results

    repo, cwd, test_executable, _ = setup_sample(sample, None)

    # Run the PASS_TO_PASS test, expected to PASS
    if sample.PASS_TO_PASS:
        tests = json.loads(sample.PASS_TO_PASS)

        # There are sometimes hundreds of tests which take ~30 minutes to all complete.
        # Since we'll check all the FAIL_TO_PASS tests, here we just want to confirm the
        # environment is set up correctly, so we sample 10 tests.
        tests = random.sample(tests, min(10, len(tests)))

        # Iterate with tqdm
        for test in tqdm.tqdm(tests, desc="PASS_TO_PASS tests", unit="test"):
            test_results["PASS_TO_PASS"]["total"] += 1
            passed, error = False, ""
            try:
                passed, error = get_test_result(test, cwd, test_executable)
                if passed:
                    test_results["PASS_TO_PASS"]["passed"] += 1
                elif error:
                    raise SampleError(error)
            except SampleError as e:
                test_results["PASS_TO_PASS"]["errors"].append({"test": test, "error": str(e)})
    print("PASS_TO_PASS results: ", test_results["PASS_TO_PASS"])

    # Apply test patch
    if sample.test_patch:
        print("Applying test patch...")
        apply_diff_to_repo(sample.test_patch, repo)

    # Run FAIL_TO_PASS tests expected to FAIL
    if sample.FAIL_TO_PASS:
        tests = json.loads(sample.FAIL_TO_PASS)
        for test in tqdm.tqdm(tests, desc="FAIL_TO_PASS tests", unit="test"):
            test_results["FAIL_TO_PASS_PRE"]["total"] += 1
            passed, error = False, ""
            try:
                passed, error = get_test_result(test, cwd, test_executable)
                if passed:
                    test_results["FAIL_TO_PASS_PRE"]["passed"] += 1
                elif error:
                    raise SampleError(error)
            except SampleError as e:
                test_results["FAIL_TO_PASS_PRE"]["errors"].append({"test": test, "error": str(e)})
    print("FAIL_TO_PASS_PRE results: ", test_results["FAIL_TO_PASS_PRE"])

    # Apply golden patch
    if sample.diff_edit:
        print("Applying diff_edit...")
        apply_diff_to_repo(sample.diff_edit, repo)

    # Run FAIL_TO_PASS tests expected to PASS
    if sample.FAIL_TO_PASS:
        tests = json.loads(sample.FAIL_TO_PASS)
        for test in tqdm.tqdm(tests, desc="FAIL_TO_PASS tests", unit="test"):
            test_results["FAIL_TO_PASS_POST"]["total"] += 1
            passed, error = False, ""
            try:
                passed, error = get_test_result(test, cwd, test_executable)
                if passed:
                    test_results["FAIL_TO_PASS_POST"]["passed"] += 1
                elif error:
                    raise SampleError(error)
            except SampleError as e:
                test_results["FAIL_TO_PASS_POST"]["errors"].append({"test": test, "error": str(e)})
    print("FAIL_TO_PASS_POST results: ", test_results["FAIL_TO_PASS_POST"])

    return test_results



from __future__ import annotations

import asyncio
import logging
from abc import ABC, abstractmethod
from asyncio import Event
from pathlib import Path
from typing import AsyncIterator

import attr
from openai.types.chat.completion_create_params import ResponseFormat

from mentat.code_file_manager import CodeFileManager
from mentat.errors import ModelError
from mentat.llm_api_handler import chunk_to_lines
from mentat.parsers.change_display_helper import (
    DisplayInformation,
    FileActionType,
    get_file_name_display,
    get_later_lines,
    get_previous_lines,
    get_removed_lines,
)
from mentat.parsers.file_edit import FileEdit
from mentat.parsers.streaming_printer import FormattedString, StreamingPrinter
from mentat.session_context import SESSION_CONTEXT
from mentat.utils import convert_string_to_asynciter


@attr.define
class ParsedLLMResponse:
    full_response: str = attr.field()
    conversation: str = attr.field()
    file_edits: list[FileEdit] = attr.field()
    interrupted: bool = attr.field(default=False)


class Parser(ABC):
    def __init__(self):
        self.shutdown = Event()
        self._silence_printer = False

    @abstractmethod
    def get_system_prompt(self) -> str:
        pass

    def response_format(self) -> ResponseFormat:
        return ResponseFormat(type="text")

    async def stream_and_parse_llm_response(self, response: AsyncIterator[str]) -> ParsedLLMResponse:
        """
        This general parsing structure relies on the assumption that all formats require three types of lines:
        1. 'conversation' lines, which are streamed as they come,
        2. 'special' lines, that are never shown to the user and contain information such as the file_name
        3. 'code' lines, which are the actual code written and are shown to the user in a special format
        To make a parser that differs from these assumptions, override this method instead of the helper methods
        """
        session_context = SESSION_CONTEXT.get()
        stream = session_context.stream
        code_file_manager = session_context.code_file_manager

        printer = StreamingPrinter()
        if self._silence_printer:
            printer_task = None
        else:
            printer_task = asyncio.create_task(printer.print_lines())
        message = ""
        conversation = ""
        file_edits = dict[Path, FileEdit]()

        cur_line = ""
        prev_block = ""
        cur_block = ""
        display_information: DisplayInformation | None = None
        file_edit: FileEdit | None = None
        line_printed = False
        in_special_lines = False
        in_code_lines = False
        in_conversation = True
        rename_map = dict[Path, Path]()
        interrupted = False
        async for chunk in response:
            if self.shutdown.is_set():
                interrupted = True
                printer.shutdown_printer()
                if printer_task is not None:
                    await printer_task
                stream.send("\nInterrupted by user. Using the response up to this point.")
                break

            for content in chunk_to_lines(chunk):
                if not content:
                    continue
                message += content
                cur_line += content

                # Print if not in special lines and line is confirmed not special
                if not in_special_lines:
                    if not line_printed:
                        if not self._could_be_special(cur_line):
                            line_printed = True
                            if not in_code_lines or display_information is None:
                                printer.add_string(cur_line, end="")
                                conversation += cur_line
                            else:
                                printer.add_string(
                                    self._code_line_beginning(display_information, cur_block),
                                    end="",
                                )
                                printer.add_string(
                                    self._code_line_content(
                                        display_information,
                                        cur_line,
                                        cur_line,
                                        cur_block,
                                    ),
                                    end="",
                                )
                    else:
                        if not in_code_lines or display_information is None:
                            printer.add_string(content, end="")
                            conversation += content
                        else:
                            printer.add_string(
                                self._code_line_content(display_information, content, cur_line, cur_block),
                                end="",
                            )

                # If we print non code lines, we want to reprint the file name of the next change,
                # even if it's the same file as the last change
                if not in_code_lines and not in_special_lines and line_printed:
                    printer.cur_file = None
                    printer.cur_file_display = None
                    file_edit = None
                    display_information = None
                    in_conversation = True

                # New line handling
                if "\n" in cur_line:
                    # Now that full line is in, give _could_be_special full line (including newline)
                    # and see if it should be printed or not
                    if not in_special_lines and not line_printed and not self._could_be_special(cur_line):
                        if not in_code_lines or display_information is None:
                            printer.add_string(cur_line, end="")
                        else:
                            printer.add_string(
                                self._code_line_beginning(display_information, cur_block),
                                end="",
                            )
                            printer.add_string(
                                self._code_line_content(display_information, cur_line, cur_line, cur_block),
                                end="",
                            )
                        line_printed = True

                    if self._starts_special(cur_line.strip()):
                        in_special_lines = True

                    if in_special_lines or in_code_lines:
                        cur_block += cur_line

                    if in_special_lines and self._ends_special(cur_line.strip()):
                        previous_file = None if file_edit is None else file_edit.file_path
                        previous_file_had_edits = (
                            False
                            if file_edit is None
                            else file_edit.replacements or file_edit.is_creation or file_edit.is_deletion
                        )

                        try:
                            (
                                display_information,
                                file_edit,
                                in_code_lines,
                            ) = self._special_block(
                                code_file_manager,
                                session_context.cwd,
                                rename_map,
                                cur_block,
                            )
                        except ModelError as e:
                            printer.add_string((str(e), {"color": "red"}))
                            printer.add_string("Using existing changes.")
                            printer.wrap_it_up()
                            if printer_task is not None:
                                await printer_task
                            logging.debug("LLM Response:")
                            logging.debug(message)
                            return ParsedLLMResponse(
                                message,
                                conversation,
                                [file_edit for file_edit in file_edits.values()],
                            )

                        # Rename map handling
                        if file_edit.rename_file_path is not None:
                            rename_map[file_edit.rename_file_path] = file_edit.file_path
                        if file_edit.file_path in rename_map:
                            file_edit.file_path = session_context.cwd / rename_map[file_edit.file_path]

                        # Add a delimiter directly before a new file edit if it's the same file as before
                        # This way, we get delimiters between every edit but not before or after the whole thing.
                        if previous_file == file_edit.file_path and previous_file_had_edits:
                            printer.add_delimiter()

                        printer.cur_file = str(file_edit.file_path)
                        printer.cur_file_display = get_file_name_display(display_information)
                        in_special_lines = False
                        prev_block = cur_block
                        cur_block = ""

                        # New file_edit creation and merging
                        if file_edit.file_path not in file_edits:
                            file_edits[file_edit.file_path] = file_edit
                        else:
                            cur_file_edit = file_edits[file_edit.file_path]
                            cur_file_edit.is_creation = cur_file_edit.is_creation or file_edit.is_creation
                            cur_file_edit.is_deletion = cur_file_edit.is_deletion or file_edit.is_deletion
                            if file_edit.rename_file_path is not None:
                                cur_file_edit.rename_file_path = file_edit.rename_file_path
                            cur_file_edit.replacements.extend(file_edit.replacements)
                            file_edit = cur_file_edit

                        # Send empty string to start filename block; needed in case it's a rename,
                        # in which case this is all that will be sent from this fileedit)
                        if (
                            in_conversation
                            or display_information.file_action_type == FileActionType.RenameFile
                            or (file_edit.file_path != previous_file)
                        ):
                            in_conversation = False
                            printer.add_string("", end="", allow_empty=True)

                        # Print previous lines, removed block, and possibly later lines
                        if in_code_lines or display_information.removed_block:
                            printer.add_string(get_previous_lines(display_information))
                            printer.add_string(get_removed_lines(display_information))
                            if not in_code_lines:
                                printer.add_string(get_later_lines(display_information))
                    elif in_code_lines and self._ends_code(cur_line.strip()):
                        # Adding code lines to previous file_edit and printing later lines
                        if display_information is not None and file_edit is not None:
                            self._add_code_block(
                                code_file_manager,
                                rename_map,
                                prev_block,
                                cur_block,
                                display_information,
                                file_edit,
                            )
                            printer.add_string(get_later_lines(display_information))

                        in_code_lines = False
                        prev_block = cur_block
                        cur_block = ""
                    line_printed = False
                    cur_line = ""
        else:
            # If the model doesn't close out the code lines, we might as well do it for it
            if in_code_lines and display_information is not None and file_edit is not None:
                self._add_code_block(
                    code_file_manager,
                    rename_map,
                    prev_block,
                    cur_block,
                    display_information,
                    file_edit,
                )
                printer.add_string(get_later_lines(display_information))

            # Only finish printing if we don't quit from ctrl-c
            printer.wrap_it_up()
            if printer_task is not None:
                await printer_task

        logging.debug("LLM Response:")
        logging.debug(message)
        return ParsedLLMResponse(
            message,
            conversation,
            [file_edit for file_edit in file_edits.values()],
            interrupted,
        )

    # Ideally this would be called in this class instead of subclasses
    def _get_file_lines(
        self,
        code_file_manager: CodeFileManager,
        rename_map: dict[Path, Path],
        abs_path: Path,
    ) -> list[str]:
        path = rename_map.get(
            abs_path,
            abs_path,
        )
        return code_file_manager.file_lines.get(path, []).copy()

    # These methods aren't abstract, since most parsers will use this implementation, but can be overriden easily
    def provide_line_numbers(self) -> bool:
        return True

    def line_number_starting_index(self) -> int:
        return 1

    def _code_line_beginning(self, display_information: DisplayInformation, cur_block: str) -> FormattedString:
        """
        The beginning of a code line; normally this means printing the + prefix
        """
        return (
            "+" + " " * (display_information.line_number_buffer - 1),
            {"color": "green"},
        )

    def _code_line_content(
        self,
        display_information: DisplayInformation,
        content: str,
        cur_line: str,
        cur_block: str,
    ) -> FormattedString:
        """
        Part of a code line; normally this means printing in green
        """
        return (content, {"color": "green"})

    # These methods must be overriden if using the default stream and parse function
    def _could_be_special(self, cur_line: str) -> bool:
        """
        Returns if this current line could be a special line and therefore shouldn't be printed yet.
        Once line is completed, will include a newline character.
        """
        raise NotImplementedError()

    def _starts_special(self, line: str) -> bool:
        """
        Determines if this line begins a special block
        """
        raise NotImplementedError()

    def _ends_special(self, line: str) -> bool:
        """
        Determines if this line ends a special block
        """
        raise NotImplementedError()

    def _special_block(
        self,
        code_file_manager: CodeFileManager,
        cwd: Path,
        rename_map: dict[Path, Path],
        special_block: str,
    ) -> tuple[DisplayInformation, FileEdit, bool]:
        """
        After finishing special block, return DisplayInformation to print, FileEdit to add/merge to list,
        and if a code block follows this special block.
        """
        raise NotImplementedError()

    def _ends_code(self, line: str) -> bool:
        """
        Determines if this line ends a code block
        """
        raise NotImplementedError()

    def _add_code_block(
        self,
        code_file_manager: CodeFileManager,
        rename_map: dict[Path, Path],
        special_block: str,
        code_block: str,
        display_information: DisplayInformation,
        file_edit: FileEdit,
    ) -> None:
        """
        Using the special block, code block and display_information, edits the FileEdit to add the new code block.
        """
        raise NotImplementedError()

    async def parse_llm_response(self, response: str) -> ParsedLLMResponse:
        self._silence_printer = True
        async_iter_response = convert_string_to_asynciter(response, chunk_size=100)
        parsed_response = await self.stream_and_parse_llm_response(async_iter_response)
        self._silence_printer = False
        return parsed_response

    def file_edits_to_llm_message(self, parsedLLMResponse: ParsedLLMResponse) -> str:
        raise NotImplementedError()



import asyncio
import json
import os
from datetime import datetime
from pathlib import Path
from typing import Any

from benchmarks.arg_parser import common_benchmark_parser
from benchmarks.run_sample import setup_sample
from benchmarks.swe_bench_runner import SWE_BENCH_SAMPLES_DIR, get_swe_samples
from mentat import Mentat
from mentat.config import Config
from mentat.sampler.sample import Sample
from mentat.session_context import SESSION_CONTEXT


def _score(predicted: set[Path], expected: set[Path]) -> dict[str, Any]:
    true_positives = predicted.intersection(expected)
    false_positives = predicted.difference(expected)
    false_negatives = expected.difference(predicted)
    precision = len(true_positives) / (len(true_positives) + len(false_positives))
    recall = len(true_positives) / (len(true_positives) + len(false_negatives))
    return {"precision": precision, "recall": recall, "n_true": len(expected)}


async def run_auto_context_benchmark(
    sample: Sample, config: Config, cwd: Path | str | None = None, include_context: bool = False
) -> dict[str, Any]:
    """Run a sample using Mentat and return the resulting diff"""
    starting_dir = Path.cwd()

    if not config.auto_context_tokens or not sample.context:
        raise ValueError(
            "In order to run the auto-context benchmark, sample.context must not "
            "be empty (ground truth) and config.auto_context_tokens must be > 0."
        )
    paths = [] if not include_context else [Path(a) for a in sample.context]

    try:
        _, cwd, _, _ = setup_sample(sample, None, skip_test_exec=True)
        exclude_paths = [cwd / ".venv"]
        mentat = Mentat(cwd=cwd, paths=paths, exclude_paths=exclude_paths, config=config or Config())
        await mentat.startup()
        await asyncio.sleep(0.01)  # Required to initialize llm_api_handler for embeddings

        # TODO: If there's a conversation history, we might consider the cumulative context.
        # Setup a mock for the LLM response and run the conversation until this point.
        code_context = SESSION_CONTEXT.get().code_context
        _ = await code_context.get_code_message(0, sample.message_prompt)
        predicted = set(path.relative_to(cwd) for path in code_context.include_files.keys())
        actual = {Path(a) for a in sample.context}
        score = _score(predicted, actual)

        await mentat.shutdown()
        return score
    finally:
        os.chdir(starting_dir)


def main(user_samples: list[str], directory: str):
    # Load benchmarks
    dir_path = Path(directory).resolve()
    assert dir_path.exists(), f"Invalid directory: {directory}"
    print(f"Running benchmarks from {dir_path}")
    samples: list[Sample] = []
    for root, dirs, files in os.walk(dir_path):
        for file in files:
            path = Path(root) / file
            if file.endswith(".json"):
                sample = Sample.load(path)
            else:
                continue
            if user_samples and not any(s in sample.title for s in user_samples):
                continue
            samples.append(sample)
    print("Found Samples:\n" + "\n".join(s.title for s in samples))
    print("*" * 80)

    config = Config(auto_context_tokens=8000)
    timestamp = datetime.now().strftime("%Y%m%d-%H%M%S")
    results_path = dir_path / f"context_benchmark_results_{timestamp}.jsonl"
    for sample in samples:
        print(f"Running benchmark for {sample.title}")
        accuracy = asyncio.run(run_auto_context_benchmark(sample, config, cwd=dir_path))
        print(f"Results: {accuracy}")
        print("*" * 80)
        with open(results_path, "a") as f:
            f.write(json.dumps({sample.id: accuracy}) + "\n")


if __name__ == "__main__":
    parser = common_benchmark_parser()
    args = parser.parse_args()
    if args.swe_bench:
        if args.swe_bench not in {"dev", "train", "test"}:
            print("Invalid SWE-Bench split.")
            exit(1)
        # Download and save SWE benchmarks as Samples
        samples = get_swe_samples(args.swe_bench, args.max_benchmarks)
        sample_titles = [sample.title for sample in samples]
        args.benchmarks = sample_titles
        args.directory = SWE_BENCH_SAMPLES_DIR / args.swe_bench
    main(
        args.benchmarks,
        args.directory,
    )



import logging
from dataclasses import dataclass

from spice import SpiceResponse

from mentat.llm_api_handler import model_price_per_1000_tokens
from mentat.session_context import SESSION_CONTEXT


@dataclass
class CostTracker:
    total_tokens: int = 0
    total_cost: float = 0

    last_api_call: str = ""

    def log_api_call_stats(
        self,
        response: SpiceResponse,
    ) -> None:
        decimal_places = 2

        model = response.call_args.model
        input_tokens = response.input_tokens
        output_tokens = response.output_tokens
        total_time = response.total_time

        speed_and_cost_string = ""
        self.total_tokens += response.total_tokens
        if output_tokens > 0:
            tokens_per_second = output_tokens / total_time
            speed_and_cost_string += f"Speed: {tokens_per_second:.{decimal_places}f} tkns/s"
        cost = model_price_per_1000_tokens(model)
        if cost:
            prompt_cost = (input_tokens / 1000) * cost[0]
            sampled_cost = (output_tokens / 1000) * cost[1]
            call_cost = prompt_cost + sampled_cost
            self.total_cost += call_cost
            if speed_and_cost_string:
                speed_and_cost_string += " | "
            speed_and_cost_string += f"Cost: ${call_cost:.{decimal_places}f}"

        costs_logger = logging.getLogger("costs")
        costs_logger.info(speed_and_cost_string)
        self.last_api_call = speed_and_cost_string

    def log_embedding_call_stats(self, tokens: int, model: str, total_time: float):
        cost = model_price_per_1000_tokens(model)
        # TODO: handle unknown models better / port to spice
        if cost is None:
            return

        cost = cost[0]
        call_cost = (tokens / 1000) * cost
        self.total_cost += call_cost
        costs_logger = logging.getLogger("costs")
        costs_logger.info(f"Cost: ${call_cost:.2f}")
        self.last_api_call = f"Embedding call time and cost: {total_time:.2f}s, ${call_cost:.2f}"

    def display_last_api_call(self):
        """
        Used so that places that call the llm can print the api call stats after they finish printing everything else.
        The api call will not be logged if it gets interrupted!
        """
        ctx = SESSION_CONTEXT.get()
        if self.last_api_call:
            ctx.stream.send(self.last_api_call, style="info")

    def log_whisper_call_stats(self, seconds: float):
        self.total_cost += seconds * 0.0001
